<?php
 // file: class.notebook.php
 // desc: PHP "clone" of GTK notebook widget
 // code: jeff b <jeff@univrel.pr.uconn.edu>
 // lic : GPL, v2

if (!defined(__CLASS_NOTEBOOK_PHP__)) {

define (__CLASS_NOTEBOOK_PHP__, true);

// shorthand macros
define (NOTEBOOK_NO_OPTIONS,	0);
define (NOTEBOOK_COMMON_BAR,	1);
define (NOTEBOOK_STRETCH,	2);

// recursive depth
$NOTEBOOK_DEPTH = 0;

class notebook {

  // *** VARIABLES ***
  var $psize;      // size of notebook
  var $page;       // array of pages
  var $name;       // page names
  var $com_vars;   // common variables (hidden)
  var $vars;       // variables passed
  var $cur_page;   // currently displayed page
  var $location;   // location of the page
  var $FGCOLOR;    // foreground color
  var $BGCOLOR;    // background color
  var $SUBMIT;     // name of the submit button text
  var $REFRESH;    // name of the refresh button text
  var $common_bar; // common submit/refresh bar at bottom of boxes?
  var $spill_at;   // number to spill over to next row at...
  var $stretch;    // stretch to 100% of available area?
  var $actionvar;  // action variable
  var $formdisable;// disable form tags

  // *** METHODS ***
  function notebook ($com_vars = "", $options = 0, $spill_at = false) {
    global $NOTEBOOK_DEPTH;          // get recursive depth
    $this->psize      = 0;           // start notebook size at 0

    list($this->location, $the_rest) = // initialize location 
     explode ("?", basename($GLOBALS["REQUEST_URI"]));

    $this->FGCOLOR    = "#ffffff";   // default foreground
    $this->BGCOLOR    = "#c5c5c5";   // default background
    $this->SUBMIT     = "Submit";    // submit button text
    $this->REFRESH    = "Refresh";   // refresh button text
    $this->common_bar = ($options & NOTEBOOK_COMMON_BAR); // common bar
    $this->spill_at   = $spill_at;   // pass spilling status/number
    $this->stretch    = ($options & NOTEBOOK_STRETCH); // set stretching
    if (!empty ($com_vars)) {
      $this->vars[0]  = $com_vars;
      $this->com_vars = true;
    } else { // if there are no common variables
      $this->com_vars = false;
    } // end checking for common variables
    
    // check for recursion
    if ($NOTEBOOK_DEPTH == 0) {
      $this->formdisable = false;	// make sure forms are enabled
      $NOTEBOOK_DEPTH++;		// increase level of recursion
      $this->actionvar="__action";	// set default actionvar
    } else {
      $this->formdisable = true;	// disable form tags for embedded
      $NOTEBOOK_DEPTH++;		// increase level of recursion
      $this->actionvar="__action";	// start with __action
      for ($i=1;$i<=$NOTEBOOK_DEPTH;$i++)
        $this->actionvar = "_" . $this->actionvar;
    } // end if
  } // end constructor notebook

  // function notebook->get_current_page()
  //   returns the current page of the notebook from global scope
  function get_current_page ($null_var = "") {
    global ${$this->actionvar};
    $found = 0;
    for ($i=1;$i<=($this->psize);$i++) {
      if ($this->name[$i] == ${$this->actionvar}) $found = $i;
    }  
    return $found;
  } // end function notebook->get_current_page

  // function notebook->is_done ()
  //   returns true if the form is ready to be processed
  function is_done ($null_var = "") {
    global ${$this->actionvar};
    return (${$this->actionvar} == $this->SUBMIT);
  } // end function notebook->is_done

  // function notebook->add_page (name, array of variables, text)
  //   adds a new "page" to the end of the notebook
  function add_page ($page_name, $page_vars, $page_text) {
    if ($page_name == $this->SUBMIT)
      DIE ("notebook->add_page :: name ($page_name) is same as submit name");
    if ($page_name == $this->REFRESH)
      DIE ("notebook->add_page :: name ($page_name) is same as refresh name");
    // FIXME: check for other pages with the same name
    $this->psize++; // increment counter
    $this->name[($this->psize)] = $page_name;
    $this->vars[($this->psize)] = $page_vars;
    $this->page[($this->psize)] = $page_text;
    // remove crap from the name
    $this->name[($this->psize)] = eregi_replace (
      "<[A-Z/]*>", "", $this->name[($this->psize)]);
  } // end function add_page

  // function notebook->display ()
  //   displays the current view of the notebook
  function display ($null_var = "") {
    global ${$this->actionvar}, ${$this->actionvar."_last"};

    // refresh here
    if (${$this->actionvar} == $this->REFRESH)
      ${$this->actionvar} = ${$this->actionvar."_last"};

    // provision for embedded table
    if (empty(${$this->actionvar}) and (!empty(${$this->actionvar."_last"})))
      ${$this->actionvar} = ${$this->actionvar."_last"};

    // here we decide on the current page, and display it
    $this->cur_page = $this->get_current_page(); // grab current page
    if ($this->cur_page < 1) $this->cur_page = 1; // bounds checking
    $buffer .= "
      ".( (!$this->formdisable) ?
       "<FORM ACTION=\"".$this->location."\" METHOD=POST>" : "" )."
      <INPUT TYPE=HIDDEN NAME=\"".$this->actionvar."_last\"
       VALUE=\"".prepare(${$this->actionvar})."\">
      <TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"".
       $this->FGCOLOR."\" ".( ($this->stretch) ? "WIDTH=100%" : "").">
    ".$this->display_notebook_bar($this->cur_page)."
    ".$this->display_hidden_variables($this->cur_page)."
      <TR><TD COLSPAN=\"".( $this->spill_at ? $this->spill_at : $this->psize)
        ."\" ".($this->stretch ? "ALIGN=CENTER" : "" ).">
	".($this->stretch ? "<CENTER>" : "" )."
      <!-- notebook body begins here -->
      ";
    $buffer .= $this->page[($this->cur_page)];
    if ($this->common_bar) $buffer .= "
	".($this->stretch ? "</CENTER>" : "" )."
      </TD></TR><TR><TD COLSPAN=\"".
        ( $this->spill_at ? $this->spill_at : $this->psize)."\"
                     BGCOLOR=\"".$this->BGCOLOR."\">
       <CENTER>
       ".$this->generate_submit()."
       &nbsp;
       ".$this->generate_refresh()."
       </CENTER>
      </TD></TR>
      ";
    $buffer .= "
      <!-- notebook body ends here -->
      </TD></TR></TABLE>".( (!$this->formdisable) ? "</FORM>" : "" )."
    ";

    return $buffer;
  } // end function notebook->display

  // function notebook->display_notebook_bar (page number)
  //   displays the notebook navigation bar at the top
  function display_notebook_bar($page_to_display) {
    $buffer = ""; // initialize buffer
    unset ($bars);
    $cur_bar = 1;
    $bars[$cur_bar] = "\n<TR>\n";
    for ($i=1;$i<=$this->psize;$i++) { // loop through pages
      if (($this->spill_at) AND
          ((($i - 1) % $this->spill_at) == 0) AND
	  ($i != 1)) {
	$bars[$cur_bar] .= "\n</TR>\n";
	$cur_bar++;
        $bars[$cur_bar] = "\n<TR>\n";
      } // end checking for spill
      $bars[$cur_bar] .= "
        <TD COLSPAN=1 ALIGN=LEFT VALIGN=BOTTOM
	 ".( ($i == $page_to_display) ?
	   "BGCOLOR=\"".$this->FGCOLOR."\"" :
	   "BGCOLOR=\"".$this->BGCOLOR."\"" ).">
         <INPUT TYPE=SUBMIT NAME=\"".$this->actionvar."\" VALUE=\"".
	 htmlentities($this->name[$i]).
	 "\">
	</TD>
      ";
      if ($i == $page_to_display) $active_bar = $cur_bar;
    } // end looping through pages
    if (($this->spill_at) AND
        (($this->psize % $this->spill_at) > 0)) {
      $bars[$cur_bar] .= "\n<TD COLSPAN=".
        ($this->spill_at - ($this->psize % $this->spill_at))." ".
           "ALIGN=LEFT VALIGN=BOTTOM BGCOLOR=\"".$this->BGCOLOR."\">".
	   "&nbsp;</TD>\n";
    } // end checking for spill
    $bars[$cur_bar] .= "\n</TR>\n";

    // display bars
    for ($i=1;$i<=$cur_bar;$i++) {
      // if it isn't the active bar, display it
      if ($i != $active_bar) $buffer .= $bars[$i];
    } // loop through all the bars

    // now display active bar
    $buffer .= $bars[$active_bar];

    return $buffer;
  } // end function notebook->display_notebook_bar

  // function notebook->display_hidden_variables (page number)
  //   embeds all of the variables into the page in INPUT TYPE=HIDDEN
  //   tags, skipping over the current page. handles all types, including
  //   arrays.
  function display_hidden_variables($__page_to_display) {
    $buffer = ""; // initialize buffer
    // loop through pages
    for ($__i=($this->com_vars ? 0 : 1);$__i<=($this->psize);$__i++) {
      if (($__page_to_display != $__i) and
          (!empty($this->vars[$__i]))) { // skip current page and empties
        $__this_set = flatten_array ($this->vars[$__i]);
        for ($__j=0;$__j<(count($__this_set));$__j++) { // loop vars
	  $__inner_set = $__this_set [$__j];
          global $$__inner_set; // pull into local scope
	  if (!is_array($$__inner_set)) {
            // if it is just scalar, display it
	    if (!empty($$__inner_set))
	      $buffer .= "<INPUT TYPE=HIDDEN NAME=\"".
	        htmlentities(stripslashes($__inner_set)).
	        "\" VALUE=\"".
	        htmlentities(stripslashes($$__inner_set)).
	        "\">\n";
	  } else { // if it *is* an array
            for ($__k=0;$__k<(count($$__inner_set));$__k++) {
	      $buffer .= "<INPUT TYPE=HIDDEN NAME=\"".
	        htmlentities(stripslashes($__inner_set)).
	        "[]\" VALUE=\"".
	        htmlentities(stripslashes(${$__inner_set}[$__k])).
	        "\">\n";
	    } // end looping through array
	  } // end checking for array
	} // end looping through variables
      } // end checking for current page
    } // end looping through pages
    return $buffer;
  } // end function notebook->display_hidden_variables

  function set_foreground_color ($new_color) {
    $this->FGCOLOR = $new_color;
  } // end function notebook->set_foreground_color

  function set_background_color ($new_color) {
    $this->BGCOLOR = $new_color;
  } // end function notebook->set_background_color

  function set_submit_name ($new_text) {
    $this->SUBMIT = $new_text;
  } // end function notebook->set_submit_name

  function set_refresh_name ($new_text) {
    $this->REFRESH = $new_text;
  } // end function notebook->set_refresh_name

  function set_spill_at ($spill_at) {
    $this->spill_at = $spill_at;
  } // end function notebook->set_spill_at

  function set_stretch_on ($null_var="") {
    $this->stretch = true;
  } // end function notebook->set_stretch_on

  function set_stretch_off ($null_var="") {
    $this->stretch = false;
  } // end function notebook->set_stretch_off

  function generate_submit ($null_val = "") {
    return " <INPUT TYPE=SUBMIT NAME=\"".$this->actionvar."\" ".
      "VALUE=\"".htmlentities($this->SUBMIT)."\">\n";
  } // end function generate_submit

  function generate_refresh ($null_val = "") {
    return " <INPUT TYPE=SUBMIT NAME=\"".$this->actionvar."\" ".
      "VALUE=\"".htmlentities($this->REFRESH)."\">\n";
  } // end function generate_submit

} // end class notebook


} // end checking if defined

?>
