<?php
 // file: class.sql.php
 // desc: generic SQL class for PHP
 // code: jeff b <jeff@univrel.pr.uconn.edu>
 // lic : GPL, v2

if (!defined (__CLASS_SQL_PHP__)) {

define (__CLASS_SQL_PHP__, true);

// database engines' shorthand
define (SQL_MSQL,		1);
define (SQL_MSSQL,		2);
define (SQL_MYSQL, 		3);
define (SQL_ODBC,  		4);
define (SQL_ORACLE,		5);
define (SQL_POSTGRES,		6);

// var types
define ( SQL_BIT_SHIFT,			4);

define ( SQL_BLOB,			1);
function SQL_CHAR ($size) 		{ return 2 + ($size<<SQL_BIT_SHIFT); }
define ( SQL_DATE,			3);
function SQL_DOUBLE ($size)		{ return 4 + ($size<<SQL_BIT_SHIFT); }
function SQL_INT ($size)		{ return 5 + ($size<<SQL_BIT_SHIFT); }
function SQL_INT_UNSIGNED ($size)	{ return 6 + ($size<<SQL_BIT_SHIFT); }
define ( SQL_TEXT,			7);
function SQL_VARCHAR ($size) 		{ return 8 + ($size<<SQL_BIT_SHIFT); }

class sql {
  var $Connection;		// connection variable
  var $SQL_Seek_Position;	// position pointer
  var $DB_ENGINE;		// current database engine
  var $VB;			// verbose?

  // constructor sql
  function sql ($type, $host, $user, $password, $database, $verbose=true) {
    $this->DB_ENGINE = $type;
    $this->VB        = $verbose;
    switch ($type) {
      case SQL_MSQL:
        $this->Connection = msql_pconnect ($host) or
	  DIE ("sql->constructor(SQL_MSQL) : could not connect");
	msql_select_db ($database, $this->Connection) or
	  DIE ("sql->constructor(SQL_MSQL) : could not select database");
        break; // end SQL_MSQL

      case SQL_MSSQL:
        $this->Connection = mssql_pconnect ($host, $user, $password) or
	  DIE ("sql->constructor(SQL_MSSQL) : could not connect");
	mssql_select_db ($database) or
	  DIE ("sql->constructor(SQL_MSSQL) : could not select database");
        break; // end SQL_MSSQL
	
      case SQL_MYSQL:
        $this->Connection = mysql_pconnect ($host, $user, $password) or
	  DIE ("sql->constructor(SQL_MYSQL) : could not connect");
	mysql_select_db ($database) or
	  DIE ("sql->constructor(SQL_MYSQL) : could not select database");
        break; // end SQL_MYSQL
	
      case SQL_ODBC:
        $this->Connection = odbc_pconnect ($host, $user, $password) or
	  DIE ("sql->constructor(SQL_ODBC) : could not connect");
	odbc_autocommit ($this->Connection, true) or  
	  DIE ("sql->constructor(SQL_ODBC) : could not set autocommit");
        break; // end SQL_ODBC

      case SQL_ORACLE:
        $this->Connection = OCIPLogon ($user."@".$host, $password, $database) or
	  DIE ("sql->constructor(SQL_ORACLE) : could not connect");
	Ora_CommitOn ($this->Connection) or  
	  DIE ("sql->constructor(SQL_ORACLE) : could not set autocommit");
        break; // end SQL_ORACLE

      case SQL_POSTGRES:
        $this->Connection = pg_pConnect ("host=$host, user=$user, ".
	                   "password=$password dbname=$database") or
	  DIE ("sql->constructor(SQL_POSTGRES) : could not connect");
        break; // end SQL_POSTGRES

      default:
        DIE("sql->constructor() : invalid database engine");
	break; // end of default
    } // end type switch
  } // end constructor sql

  function data_seek ($result, $pos) {
    // wrapper for sql->seek
    return $this->seek ($result, $pos);
  } // end function sql->data_seek

  function db_query ($database, $query) {
    switch ($this->DB_ENGINE) {
      case SQL_MSQL:
        $result = msql_db_query ($database, $query);
        break; // end SQL_MSQL

      case SQL_MSSQL:
        $result = mssql_db_query ($database, $query);
        break; // end SQL_MSSQL

      case SQL_MYSQL:
        $result = mysql_db_query ($database, $query);
        break; // end SQL_MYSQL

      case SQL_ODBC:
        $result = odbc_exec ($this->Connection, $query);
        break; // end SQL_ODBC

      case SQL_ORACLE:
        // FIXME!: probably broken
        OCIParse ($this->Connection, $query);
	OCIExecute ($result);
	OCICommit ($this->Connection);
        break; // end SQL_ORACLE

      case SQL_POSTGRES:
        $result = pg_exec ($this->Connection, $query);
        break; // end SQL_POSTGRES

      default:
        DIE("sql->db_query() : invalid database engine");
	break; // end of default
    } // end type switch

    // common code
    if ($VB and !($result)) echo "ERROR";
    $this->SQL_Seek_Position["$result"] = 0;
    return $result;
  } // end function sql->db_query

  function fetch_array ($result) {
    if ($this->num_rows($result) <= $this->SQL_Seek_Position["$result"])
      return NULL;
    switch ($this->DB_ENGINE) {
      case SQL_MSQL:
        msql_data_seek ($result, $this->SQL_Seek_Position["$result"]);
        $this_result = msql_fetch_array ($result);
        break; // end SQL_MSQL

      case SQL_MSSQL:
        mssql_data_seek ($result, $this->SQL_Seek_Position["$result"]);
        $this_result = mssql_fetch_array ($result);
        break; // end SQL_MSSQL

      case SQL_MYSQL:
        mysql_data_seek ($result, $this->SQL_Seek_Position["$result"]);
        $this_result = mysql_fetch_array ($result);
        break; // end SQL_MYSQL

      case SQL_ODBC:
        $odbc_cols = odbc_fetch_into ($result,
	  $this->SQL_Seek_Position["$result"],
	  &$raw_array);
	unset ($this_result);  
	for ($i=0;$i<$odbc_cols;$i++) {
	  $this_name = odbc_field_name ($result, $i);
          $this_result["$this_name"] = $raw_array[$i];
	} // end looping through columns
        break; // end SQL_ODBC

      case SQL_ORACLE:
        // FIXME!: this is probably *very* broken
        OCIFetchInto ($result, &$this_result, OCI_ASSOC);
        break; // end SQL_ORACLE

      case SQL_POSTGRES:
        $this_result = pg_Fetch_Array ($result,
	  $this->SQL_Seek_Position["$result"]);
        break; // end SQL_POSTGRES

      default:
        DIE("sql->fetch_array() : invalid database engine");
	break; // end of default
    } // end type switch
    
    // common stuff
    $this->SQL_Seek_Position["$result"]++;
    return $this_result;
  } // end function sql->fetch_array

  function num_rows ($result) {
    if ($result == 0) return 0;
    switch ($this->DB_ENGINE) {
      case SQL_MSQL:
        $num = msql_num_rows ($result);
        break; // end SQL_MSQL

      case SQL_MSSQL:
        $num = mssql_num_rows ($result);
        break; // end SQL_MSSQL

      case SQL_MYSQL:
        $num = mysql_num_rows ($result);
        break; // end SQL_MYSQL

      case SQL_ODBC:
        $num = odbc_num_rows ($result);
        break; // end SQL_ODBC

      case SQL_ORACLE:
        $num = OCIRowCount ($result);
        break; // end SQL_ORACLE

      case SQL_POSTGRES:
        $num = pg_Num_Rows ($result);
        break; // end SQL_POSTGRES

      default:
        DIE("sql->num_rows() : invalid database engine");
	break; // end of default
    } // end type switch

    // common stuff
    if ($VB and !($num)) echo _("ERROR");
    return $num;
  } // end function sql->num_rows

  function query ($query) {
    switch ($this->DB_ENGINE) {
      case SQL_MSQL:
        $result = msql_query ($query);
        break; // end SQL_MSQL

      case SQL_MSSQL:
        $result = mssql_query ($query);
        break; // end SQL_MSSQL

      case SQL_MYSQL:
        $result = mysql_query ($query);
        break; // end SQL_MYSQL

      case SQL_ODBC:
        $result = odbc_exec ($this->Connection, $query);
        break; // end SQL_ODBC

      case SQL_ORACLE:
        // FIXME!: probably broken
        OCIParse ($this->Connection, $query);
	OCIExecute ($result);
	OCICommit ($this->Connection);
        break; // end SQL_ORACLE

      case SQL_POSTGRES:
        $result = pg_exec ($this->Connection, $query);
        break; // end SQL_POSTGRES

      default:
        DIE("sql->query() : invalid database engine");
	break; // end of default
    } // end type switch

    // common code
    if ($VB and !($result)) echo "ERROR";
    $this->SQL_Seek_Position["$result"] = 0;
    return $result;
  } // end function sql->query

  // hacked function to true|false if there are results
  function results ($result) {
    return ($result and ($this->num_rows($result)>0));
  } // end function sql->result

  function seek ($result, $pos) {
    // move internal pointer
    $this->SQL_Seek_Position["$result"] = $pos;
  } // end function sql->seek

  /* SQL query wrappers ------------------------------------------------ */

  function create_table_query ($table, $values, $keys=NULL) {
    // clear query
    $query = "";

    // produce first portion
    $query .= "CREATE TABLE ".addslashes($table)." ( ";

    // loop through values
    reset ($values); $in_loop = false;
    while (list ($k, $v) = each ($values)) {

      // if there is a bit shifted portion, get that
      $bit_shifted = (int)($v>>SQL_BIT_SHIFT);
    
      // build depending on params
      switch (($v % pow(2,SQL_BIT_SHIFT))) {
      
        case SQL_BLOB:
         $query .= ( $in_loop ? ", " : " " ).addslashes($k)." BLOB";
	 break; // end SQL_BLOB

	case SQL_CHAR(0):
         $query .= ( $in_loop ? ", " : " " ).addslashes($k).
	  " CHAR(".($bit_shifted + 0).")";
	 break; // end SQL_CHAR

        case SQL_DATE:
         $query .= ( $in_loop ? ", " : " " ).addslashes($k)." DATE";
	 break; // end SQL_DATE

	case SQL_DOUBLE(0):
         $query .= ( $in_loop ? ", " : " " ).addslashes($k).
	  " DOUBLE". ( ($bit_shifted>0) ? "(".($bit_shifted + 0).")" : "" );
	 break; // end SQL_CHAR

	case SQL_INT(0):
	case SQL_INT_UNSIGNED(0):
         $query .= ( $in_loop ? ", " : " " ).addslashes($k).
	  " INT". ( ($bit_shifted>0) ? "(".($bit_shifted + 0).")" : "" ).
	  ( (($v % pow(2,SQL_BIT_SHIFT))==SQL_INT_UNSIGNED(0)) ?
	    " UNSIGNED" : "" );
	 break; // end SQL_{INT,INT_UNSIGNED}

	case SQL_TEXT:
	 // this maps to BLOB unless you use MySQL
         $query .= ( $in_loop ? ", " : " " ).addslashes($k)." ".
	  ( ($this->DB_ENGINE==SQL_MYSQL) ? "BLOB" : "TEXT");
	 break; // end SQL_TEXT

	case SQL_VARCHAR(0):
         $query .= ( $in_loop ? ", " : " " ).addslashes($k).
	  " ". ( ($this->DB_ENGINE==SQL_MYSQL) ? "VARCHAR" : "CHAR" ).
	  "(".($bit_shifted + 0).")";
	 break; // end SQL_VARCHAR

        default: // check for errors
	 DIE("sql->create_table_query : improper type");
	 break; // end everything else

      } // end value switch
      $in_loop = true;
    } // end while loop through values

    if (!empty($keys)) {
      $query .= ", PRIMARY KEY (".addslashes($keys[0]).")";
    } // end if not empty $keys

    // return query
    return $query;
  } // end function sql->create_table_query

  function drop_table_query ($table) {
    // produce first portion
    $query = "DROP TABLE ".addslashes($table);

    return $query;
  } // end function sql->drop_table_query

  function insert_query ($table, $values) {
    // clear query
    $query = "";

    // loop through values
    reset ($values); $in_loop = false; $values_hash = ""; $cols_hash = "";
    while (list ($k, $v) = each ($values)) {
      // decide whether we use the key/value pairs or just the value
      if ( (($k+0)>0) or (empty($k)) ) {
        $k = $v; 		// pass name pair
	$v = $GLOBALS[$v];	// grab value from GLOBALS[]
      } // end checking for key/value pairs
      
      // add to hashes
      $values_hash .= ( $in_loop ? ", " : " " ).
        "'".addslashes(sql_squash($v))."'";
      $cols_hash .= ( $in_loop ? ", " : " " ).	
        addslashes(sql_squash($k));
      $in_loop = true;
    } // end while loop through values
    
    // form query from parts
    $query .= "INSERT INTO ".addslashes($table).
      " ( ".$cols_hash." ) ".
      "VALUES ( ".$values_hash." )";

    // return formed query
    return $query;
  } // end function sql->insert_query

  function update_query ($table, $values, $where_var) {
    // clear query
    $query = "";

    // loop through values
    reset ($values); $in_loop = false; $values_hash = "";
    while (list ($k, $v) = each ($values)) {
      // decide whether we use the key/value pairs or just the value
      if ( (($k+0)>0) or (empty($k)) ) {
        $k = $v; 		// pass name pair
	$v = $GLOBALS[$v];	// grab value from GLOBALS[]
      } // end checking for key/value pairs
      
      // add to hashes
      $values_hash .= ( $in_loop ? ", " : " " ).
        $k . " = '".addslashes(sql_squash($v))."'";
      $in_loop = true;
    } // end while loop through values
    
    // figure out wherevar syntax
    if (is_array ($where_var)) { // if an array, then array (name => val)
      reset ($where_var);
      list ($where_key, $where_val) = each ($where_var);
    } else { // if no array, it must be the name of the variable
      $where_key = $where_var;
      $where_val = $GLOBALS["$where_var"];
    } // end checking
    
    // form query from parts
    $query .= "UPDATE ".addslashes($table)." ".
      "SET ".$values_hash." ".
      "WHERE ( ".$where_key." = '".addslashes($where_val)."' )";

    // return formed query
    return $query;
  } // end function sql->update_query

} // end class sql

} // end checking if defined

?>
