<?php
 // file: misc_tools.php
 // desc: miscellaneous tools
 // code: jeff b <jeff@univrel.pr.uconn.edu>
 // lic : GPL, v2

if (!defined(__MISC_TOOLS_PHP__)) {

define (__MISC_TOOLS_PHP__, true);

function web_link ($text, $link = "") {
  if ($link == "") $link = $text;
  return "<A HREF=\"$link\" 
    onMouseOver=\"window.status='$text'; return true;\"
    onMouseOut=\"window.status=''; return true;\"
    >$text</A>";
} // end function web_link

function in_this ($var, $val) {
  if (!is_array ($var)) {
    // handle as scalar
    if ($var == $val) return true;
  } else {
    // handle as array
    for ($i=0;$i<count($var);$i++) 
      if ($var[$i] == $val) return true;
  } // end checking how to handle

  // if all else fails... false
  return false;
} // end function in_this

function array_element ($array, $element) {
  return ( (is_array($array)) ? $array[$element] : $array );
} // end function array_element

// implement PHP3 version of in_array
if (floor(phpversion()) < 4) {
  function in_array ($needle, $haystack) {
    for ($i=0; $i<count($haystack) && $haystack[$i] != $needle; $i++);
    return ($i != count($haystack));
  } // end function in_array
}

function prepare ($string, $force_conversion=false) {
  return ( (!$force_conversion and eregi("<[A-Z/]*>", $string)) ?
    stripslashes($string) :
    htmlentities(stripslashes($string))
  );
} // end function prepare

function state_pulldown ($varname, $is_full=false) {
  $states = array (
    "AL" => "Alabama",
    "AK" => "Alaska",
    "AS" => "American Samoa",
    "AZ" => "Arizona",
    "AR" => "Arkansas",
    "CA" => "California",
    "CO" => "Colorado",
    "CT" => "Connecticut",
    "DE" => "Delaware",
    "DC" => "District of Columbia",
    "FM" => "Federated States of Micronesia",
    "FL" => "Florida",
    "GA" => "Georgia",
    "GU" => "Guam",
    "HI" => "Hawaii",
    "ID" => "Idaho",
    "IL" => "Illinois",
    "IN" => "Indiana",
    "IA" => "Iowa",
    "KS" => "Kansas",
    "KY" => "Kentucky",
    "LA" => "Louisiana",
    "ME" => "Maine",
    "MH" => "Marshall Islands",
    "MD" => "Maryland",
    "MA" => "Massachussetts",
    "MI" => "Michigan",
    "MN" => "Minnesota",
    "MS" => "Mississippi",
    "MO" => "Missouri",
    "MT" => "Montana",
    "NE" => "Nebraska",
    "NV" => "Nevada",
    "NH" => "New Hampshire",
    "NJ" => "New Jersey",
    "NM" => "New Mexico",
    "NY" => "New York",
    "NC" => "North Carolina",
    "ND" => "North Dakota",
    "NP" => "Northern Mariana Islands",
    "OH" => "Ohio",
    "OK" => "Oklahoma",
    "OR" => "Oregon",
    "PW" => "Palau",
    "PA" => "Pennsylvania",
    "PR" => "Puerto Rico",
    "RI" => "Rhode Island",
    "SC" => "South Carolina",
    "SD" => "South Dakota",
    "TN" => "Tennessee",
    "TX" => "Texas",
    "UT" => "Utah",
    "VT" => "Vermont",
    "VI" => "Virgin Islands",
    "VA" => "Virginia",
    "WA" => "Washington",
    "WV" => "West Virginia",
    "WI" => "Wisconsin",
    "WY" => "Wyoming"
  );
  global $$varname; // import variable

  // reset everything
  reset ($states); $buffer = "";

  // sort by the proper category
  if ($is_full) { sort ($states); } else { ksort ($states); }

  // start select box
  $buffer .= "\n<SELECT NAME=\"".prepare($varname)."\">\n";

  while (list($key, $val) = each($states)) {
    $buffer .= "<OPTION VALUE=\"".prepare($key)."\" ".
      ( in_this ($$varname, $key) ? "SELECTED" : "" ).
      ">".prepare(( $is_full ? $val : $key ))."\n";
  } // end of while looping

  // end select box
  $buffer .= "\n</SELECT>\n";

  // return buffer
  return $buffer;

} // end function state_pulldown

function country_pulldown ($varname) {
  global $$varname; // import variable
  $countries = array (
    "United States",
    "Canada"
  );

  // reset everything
  reset ($countries); $buffer = "";

  // start select box
  $buffer .= "\n<SELECT NAME=\"".prepare($varname)."\">\n";

  while (list($key, $val) = each($countries)) {
    $buffer .= "<OPTION VALUE=\"".prepare($val)."\" ".
      ( in_this ($$varname, $val) ? "SELECTED" : "" ).
      ">".prepare($val)."\n";
  } // end of while looping

  // end select box
  $buffer .= "\n</SELECT>\n";

  // return buffer
  return $buffer;

} // end function country_pulldown

function gender_select ($varname) {
  global $$varname;

  $buffer = "";

  $buffer .= "\n<INPUT TYPE=RADIO NAME=\"".prepare($varname)."\" ".
    "VALUE=\"m\" ".
    ( in_this ($$varname, "m") ? "CHECKED" : "" ).">Male &nbsp;\n".
    "<INPUT TYPE=RADIO NAME=\"".prepare($varname)."\" ".
    "VALUE=\"f\" ".
    ( in_this ($$varname, "f") ? "CHECKED" : "" ).">Female\n";

  return $buffer;
} // end function gender_select

function number_pulldown ($varname, $start, $end, $step=1, $addzero=true) {
  global $$varname;

  $buffer = "";

  // display top of selet box
  $buffer .= "\n<SELECT NAME=\"".prepare($varname)."\">\n";

  // loop
  for ($i=$start;$i<=$end;$i+=$step) {
    $buffer .= "<OPTION VALUE=\"".
      ( ($addzero and ($i<10)) ? "0" : "" )."$i\" ".
      ( in_this ($$varname, $i) ? "SELECTED" : "" ).">".
      ( ($addzero and ($i<10)) ? "0" : "" )."$i\n";
  } // end loop

  // display bottom of selet box
  $buffer .= "\n</SELECT>\n";

  return $buffer;
} // end function number_pulldown

function select_option ($varname, $value, $text="") {
  global $$varname;

  // dump value into text if nothing presented
  if (empty($text)) $text=$value;

  // empty buffer
  $buffer = "";

  // create option
  $buffer .= "<OPTION VALUE=\"".prepare($value)."\" ".
    ( in_this($$varname, $value) ? "SELECTED" : "" ).
    ">".prepare($text);

  // return buffer
  return $buffer;
} // end function select_option

function select_widget ($varname, $values, $array_index = -1) {
  global $$varname;

  // no nested arrays here!
  $this_values = flatten_array ($values);

  // clear buffer
  $buffer = "";

  // display top of selet box
  $buffer .= "\n<SELECT NAME=\"".prepare($varname).
    ( ($array_index != -1) ? "[]" : "" )."\">\n";

  reset ($this_values);
  while (list($k, $v) = each ($this_values)) {
    if ($array_index == -1) {
      $buffer .= select_option ($varname, $v, ( !empty($k) ? $k : $v ));
    } else { // if there is an index ...
      $buffer .= "<OPTION VALUE=\"".prepare($v)."\" ".
        ( ((${$varname}["$array_index"])==$v) ? "SELECTED" : "" ).
        ">".prepare(( !empty ($k) ? $k : $v ))."\n";
    } // end if
  } // end looping

  // end select box
  $buffer .= "\n</SELECT>\n";

  // return buffer
  return $buffer;
} // end function select_widget

function flatten_array ($initial_array) {
  // handle non-array parameters
  if (!is_array($initial_array)) {
    $result_array[] = $intial_array;
    return $result_array;
  } // end if not array

  // loop through entire array
  reset ($initial_array);
  $result_array = array();
  while (list($key, $val) = each ($initial_array)) {
    if (is_array($val)) {
      $sub_array = flatten_array ($val);
      $result_array = ( (is_array($result_array)) ?
        array_merge ($result_array, $sub_array) :
	$sub_array
      );
    } else { // if scalar
      if (!strlen($val)<1) { // strlen instead of empty to handle "0"s
        // if it is a non-numeric key...
        if (($key+0)==0) { $result_array["$key"] = $val; }
	  else           { $result_array[]       = $val; }
      } // end checking for empty key
    } // end checking array or scalar
  } // end of while loop

  // return result array
  asort ($result_array);
  return $result_array;
} // end function flatten_array

function unique_array ($_this_array) {
  $this_array = $_this_array;
  if (!is_array($this_array)) return $this_array;
  sort ($this_array);
  for ($i=0;$i<count($this_array);$i++) 
    if ( ($i>0) && ($this_array[$i]!=$this_array[$i-1]) )
      $result_array[$i] = $this_array[$i];
  if (is_array($result_array)) reset ($result_array);    
  return $result_array;
} // end function unique_array

function alternate_colors ($colors = "") {
  static $cur_pos;

  // default values for switching
  if (empty($colors))
    $colors = array (
      "#dddddd", "#ffffff"
    );

  // if never run before, reset current position
  if (empty($cur_pos)) { $cur_pos = 0; }
   else {
     $cur_pos += 1;
     if ($cur_pos > count($colors)) $cur_pos = 0;
   }

   return $colors[$cur_pos]; 
} // end function alternate_colors

function checkbox_widget ($varname, $value, $text="") {
  global $$varname;

  // dump value into text if nothing presented
  if (empty($text)) $text=$value;

  // empty buffer
  $buffer = "";

  // create option
  $buffer .= "<INPUT TYPE=CHECKBOX NAME=\"".prepare($varname).
    ( is_array ($$varname) ? "[]" : "" )."\" ".
    "VALUE=\"".prepare($value)."\" ".
    ( in_this($$varname, $value) ? "CHECKED" : "" ).
    ">".prepare($text)."\n";

  // return buffer
  return $buffer;
} // end function checkbox_widget

function form_table ($cells) {
  $buffer = "";

  $buffer .= "
   <CENTER>
   <TABLE WIDTH=\"100%\" CELLSPACING=0 CELLPADDING=3 ALIGN=CENTER>
  ";

  reset ($cells);
  while (list ($k, $v) = each ($cells)) {
   if (!empty ($v)) {
    $buffer .= "
      <TR>
       <TD COLSPAN=1 ALIGN=RIGHT VALIGN=MIDDLE>
        ".( (!empty($k)) ? prepare($k)." : " : "&nbsp;" )."
       </TD><TD COLSPAN=2 ALIGN=LEFT VALIGN=MIDDLE>
        ".$v."
       </TD>
      </TR>\n";
   } // end if not empty    
  } // end looping through cells

  $buffer .= "
   </TABLE>
   </CENTER>
  ";

  // return the buffer
  return $buffer;
} // end function form_table

function combo_widget ($varname, $values) {
  global $$varname, ${$varname."_text"};

  // reset buffer
  $buffer = "";

  // form select box first
  reset ($values); $select_box = ""; $max_length = 1;
  $select_box .= "    <SELECT NAME=\"$varname\">\n";
  $select_box .= "     <OPTION VALUE=\"\"> ----&gt;\n";
  while (list ($k, $v) = each ($values)) {
    if (strlen($k)>$max_length) $max_length=strlen($k);   
    $select_box .= "     <OPTION VALUE=\"".prepare($v)."\" ".
      ( ($$varname==$v) ? "SELECTED" : "" ).">".prepare($k)."\n";
  } // end looping through everything
  $select_box .= "    </SELECT>\n";
  
  $buffer .= "
    <TABLE BORDER=0 CELLSPACING=0 CELLPADDING=0>
     <TR>
      <TD>$select_box</TD>
      <TD><INPUT TYPE=TEXT NAME=\"".prepare($varname)."_text\" ".
          "SIZE=\"".$max_length."\"
           VALUE=\"".prepare(${$varname."_text"})."\">
     </TR>
    </TABLE>
  ";

  return $buffer;
} // end function combo_widget

function combo_assemble ($varname) {
  global $$varname, ${$varname."_text"};
  return ( !empty($$varname) ? $$varname : ${$varname."_text"} );
} // end function combo_assemble

function page_name ($page_name="") {
  $this_page = ( (empty($page_name)) ? $GLOBALS["REQUEST_URI"] : $page_name );
  list ($page, $garbage) = explode ("?", basename ($this_page));
  return $page;
} // end function

} // end checking if defined

?>
